#!/bin/sh
#===============================================================================================================================================
#
# Program         : IPFire Mirror
# File            : IPFire.sh
# Version         : 1.2
# Description     : Rsync IPFire Mirror Script
#
#------------------------------------------------------------------------
#
# Autor           : Michael Kleger, OneSystems GmbH
# Date            : 31.05.2015 -->     Creation Date
# Update          : 01.06.2015 -->     Log and Mail Function
#
#===============================================================================================================================================

#-----------------------------------------------------------------------------------------------------------------------------------------------
# Configuration
#-----------------------------------------------------------------------------------------------------------------------------------------------
    # Mirror Name
    NAME="IPFire"

    # Mirror server and local directorys
    SERVER=rsync.ipfire.org::full
    MIRROR=/var/www/html/ipfire/
    LOGDIR=/var/log/mirror

    # Log and Debug files
    LOGFILE=${LOGDIR}/${NAME}.log
    DEBUGFILE=${LOGDIR}/${NAME}.debug

    # Who will be informed in case if anything goes wrong (if you do not want to be informed via mail, set this option to "")
    MAILNOTIFY="info@w2k.ch"

    # Subject of the notification mail
    MAILSUBJECT="ERROR while synchronizing ${NAME} mirror"

    # Lock datei
    LOCKFILE=/var/tmp/${NAME}Sync.lock


#-----------------------------------------------------------------------------------------------------------------------------------------------
# Create local directories if not exists
#-----------------------------------------------------------------------------------------------------------------------------------------------	
	if [ ! -d ${MIRROR} ] ; then
	  mkdir -p ${MIRROR}
	fi
	if [ ! -d ${LOGDIR} ] ; then
	  mkdir -p ${LOGDIR}
	  touch $LOGFILE
	  touch $DEBUGFILE
	fi
	

#-----------------------------------------------------------------------------------------------------------------------------------------------
# Functions
#-----------------------------------------------------------------------------------------------------------------------------------------------
	function log() {
	  echo `date +%d.%m.%Y%t%H:%M:%S` "    LOG:" $1 >> ${LOGFILE}
	}

	function error() {
	  echo `date +%d.%m.%Y%t%H:%M:%S` "    ERROR:" $1 | tail -1 ${LOGFILE}
	  if [ -n "${MAILNOTIFY}" ] ; then
		echo `date +%d.%m.%Y%t%H:%M:%S` "    ERROR:" $1 | mail -s "${MAILSUBJECT}" ${MAILNOTIFY}
	  fi
	}

	function status() {
	  case "$1" in
		0)
		  log "Synchronization completed."
		  ;;
		1)
		  error "RSYNC: Syntax or usage error. (Error Code $1)"
		  ;;
		2)
		  error "RSYNC: Protocol incompatibility. (Error Code $1)"
		  ;;
		3)
		  error "RSYNC: Errors selecting input/output files, dirs. (Error Code $1)"
		  ;;
		4)
		  error "RSYNC: Requested action not supported: an attempt was made to manipulate 64-bit files on a platform that cannot support them; or an option was specified that is supported by the client and not by the server. (Error Code $1)"
		  ;;
		5)
		  error "RSYNC: Error starting client-server protocol. (Error Code $1)"
		  ;;
		6)
		  error "RSYNC: Daemon unable to append to log-file. (Error Code $1)"
		  ;;
		10)
		  error "RSYNC: Error in socket I/O. (Error Code $1)"
		  ;;
		11)
		  error "RSYNC: Error in file I/O. (Error Code $1)"
		  ;;
		12)
		  error "RSYNC: Error in rsync protocol data stream. (Error Code $1)"
		  ;;
		13)
		  error "RSYNC: Errors with program diagnostics. (Error Code $1)"
		  ;;
		14)
		  error "RSYNC: Error in IPC code. (Error Code $1)"
		  ;;
		20)
		  error "RSYNC: Received SIGUSR1 or SIGINT. (Error Code $1)"
		  ;;
		21)
		  error "RSYNC: Some error returned by waitpid(). (Error Code $1)"
		  ;;
		22)
		  error "RSYNC: Error allocating core memory buffers. (Error Code $1)"
		  ;;
		23)
		  error "RSYNC: Partial transfer due to error. (Error Code $1)"
		  ;;
		24)
		  error "RSYNC: Partial transfer due to vanished source files. (Error Code $1)"
		  ;;
		25)
		  error "RSYNC: The --max-delete limit stopped deletions. (Error Code $1)"
		  ;;
		30)
		  error "RSYNC: Timeout in data send/receive. (Error Code $1)"
		  ;;
		*)
		  error "RSYNC: Unknown error. (Error Code $1)"
		  ;;
	  esac
	}


#-----------------------------------------------------------------------------------------------------------------------------------------------
# Check if process is running
#-----------------------------------------------------------------------------------------------------------------------------------------------
	if [ -f ${LOCKFILE} ] ; then
	  kill -0 $(cat ${LOCKFILE}) >/dev/null 2>&1
	  if [ $? -eq 0 ] ; then
		error "Previous process still running."
		exit 1
	  else
		log "Deprecated lock file found. Remove lock file. (Location: ${LOCKFILE})"
		rm -f ${LOCKFILE}
	  fi
	fi

	echo $$ >${LOCKFILE}


#-----------------------------------------------------------------------------------------------------------------------------------------------
# Sync start
#-----------------------------------------------------------------------------------------------------------------------------------------------
	log "Starting ${NAME} download process."
	echo ">>>>>" `date +%d.%m.%Y%t%H:%M:%S` "<<<<<" >>${DEBUGFILE}
	rsync -avzHl --delete --progress --delay-updates --timeout=300 ${SERVER} ${MIRROR} >>${DEBUGFILE} 2>&1
	status $?

	rm -f ${LOCKFILE}

	echo $(date) > ${MIRROR}/LastUpdate.txt

	exit 0